{ ***************************************************** }
{	This program Converts levels files to useable data	}
{ ***************************************************** }

program LevelsToData;

var	Level:array[1..32,1..32] of integer;{ Storage for read level }
	Dat:array[0..16383] of byte;		{ Compressed memory space }

{*************************************************}
{*	Convert a character to its byte equivalent   *}
{*************************************************}

function ConvertData(ch:Char):Integer;
var r:Byte;
begin
	case UpCase(ch) of
		' ':	r := 0;					{ Space }
		'?':	r := 8;					{ Fake brick }
		'L':	r := 16;				{ Hidden ladder }
		'K':	r := 24;				{ Key }
		'0':    r := 32;         		{ Coin }
		'C':	r := 40;				{ Cloud }
		'-':    r := 56;        		{ Pipe }
		'H':    r := 64;                { Ladder }
		'M':    r := 72;                { Metal }
		'X':    r := 80;	            { Brick }
		'=':	r := 88;				{ Girder platform }
		'>':	r := 96;				{ Move right }
		'<':	r := 104;				{ Move left }
		'D':	r := 112;				{ Door }
		'*':	r := 48;				{ Player start }
		'E':	r := 120;				{ Bad guy }
		'W':	r := 192;				{ Warp tile }
		else
			WriteLn('Error:',ch);
	end;
	ConvertData := r;
end;

{********************************************}
{*  Compress the given file into RLE format *}
{********************************************}

procedure CompressFile(var p:Integer);
var	Start,Current,n,Count,x,y:Integer;
begin
	Start := p;
	Current := Level[1,1];				{ Start for scanning }
	Count := 1;
	x := 2;y := 1;n := 0;
	while y < 33 do						{ until finished ..... }
		begin
			if (Level[y,x] <> Current) or (Count >= 15) then
				begin
					if Current < 128 then
						Dat[p] := Count * 16 + Current div 8
					else
						Dat[p] := (Current-128) div 8;
					n := n + Count;
					Inc(p);
					Count := 0;
				end;
			Current := Level[y,x];
			Inc(Count);
			Inc(x);
			if x = 33 then
				begin
					x := 1;Inc(y);
				end;
		end;
	if Current < 128 then
			Dat[p] := Count * 16 + Current div 8
	else
			Dat[p] := (Current-128) div 8;
	n := n + Count;
	Inc(p);
    Dat[p] := 0;
	Inc(p);
	Writeln('Compressed into ',p-Start,' bytes (',(p-Start)*10 div 102,'%).');
end;

{**********************************************}
{*	Read the given file into the level array  *}
{**********************************************}

procedure ReadLevelFile(Name:String);
var i,j,LineNo:Integer;
	f:Text;
	s:String;
begin
	for i := 1 to 32 do					{ Zero the levels array }
		for j := 1 to 32 do
				Level[i,j] := 0;
	LineNo := 1;						{ on the first line }
	Assign(f,Name);						{ open the source file }
	Reset(f);
	WriteLn('Now loading "',Name,'".');
	while not eof(f) do					{ work through the file }
		begin
			ReadLn(f,s);
			for i := 1 to Length(s) do
				Level[LineNo,i] := ConvertData(s[i]);
			Inc(LineNo);
		end;
	Close(f);
end;

procedure AddLevelFile(Name:String;var Ptr:Integer);
var n:Integer;
begin
	Inc(Dat[0]);						{ Increment level count }
	n := Dat[0];
	Dat[n*2+0] := (Ptr mod 256);		{ Set up the offset }
	Dat[n*2+1] := (Ptr div 256);
	ReadLevelFile(Name);				{ Read the file }
	CompressFile(Ptr);					{ and compress it. }
end;

var i,p:Integer;
	fOut:File;
	f:Text;
	s:String;

begin
	WriteLn;
	WriteLn('Lode Runner 83 Level Compiler/Compressor');
	WriteLn;
	Dat[0] := 0;						{ +0 is the number of levels }
	Dat[1] := 0;						{ +1 is the cheat flag }
	p := 128;
	for i := 2 to p-1 do Dat[i] := 0;

	Assign(f,'LEVLIST.DAT');			{ Work through the level list }
	Reset(f);
	while not eof(f) do
		begin
			ReadLn(f,s);
			if Length(s) > 2 then AddLevelFile(s,p);
		end;
	close(f);

	Writeln('Writing ',p,' bytes to "LEVEL.BIN".');
	Assign(fOut,'LEVEL.BIN');
	Rewrite(fOut,1);
	for i := 0 to p-1 do
			BlockWrite(fOut,Dat[i],1);
	Close(fOut);
end.
